#include <string>
#include <array>
#include <stdint.h>
#include "toolbox.hpp"

static constexpr const size_t STRING_BUFFER_SIZE = 64;
static constexpr const uint8_t N_TIMES = 8;
static constexpr const char time_formats[N_TIMES + 1] = "smhjwMyc";
static constexpr const std::array<const uint64_t, N_TIMES> time_numbers = { 1, 60, 3600, 86400, 604800, 2629800, 31557600, u64(315576e4) };

/**
 * @brief Convert a given number of seconds to a human readable format
 *
 * @param time Number of seconds
 * @return Human readable formatted string
 */
std::string format_time(uint64_t time) noexcept {
	char s[STRING_BUFFER_SIZE] = { 0 };

	if(time == 0){
		sprintf(s, "0%c", time_formats[0]);
		return s;
	}

	uint64_t res;
	size_t j = 0;
	for(int8_t i = static_cast<int8_t>(time_numbers.size() - 1); i >= 0; --i) {
		if(time >= time_numbers[i]){
			res = time / time_numbers[i];
			time %= time_numbers[i];
			j += ullstr(res, j, s);
			s[j++] = time_formats[i];
			s[j++] = ' ';
		}
	}

	/* Remove trailing character */
	s[j - 1] = '\0';

	return std::string(s);
}

#ifndef TEST
int32_t main(const int32_t argc, const char* const* argv) noexcept {

	uint64_t n = 0;
	if (argc > 2) return fprintf(stderr, "Invalid usage : (%s $NUMBER) or (echo $NUMBER | %s)\n", argv[0], argv[0]);
	else if (argc == 2){
		if(sstrtoull(argv[1], n) == EXIT_FAILURE)
			return EXIT_FAILURE;
	} else {
		size_t i = 0;
		char c, BUFFER[STRING_BUFFER_SIZE];
		for(; i < STRING_BUFFER_SIZE && (c = fgetc(stdin)) != EOF; ++i)
			BUFFER[i] = c;
		if(i == STRING_BUFFER_SIZE){
			fprintf(stderr, "Error while converting to integer : invalid stdin input (too large)\n");
			return EXIT_FAILURE;
		}
		if(sstrtoull(BUFFER, n) == EXIT_FAILURE)
			return EXIT_FAILURE;
	}

	printf("%s\n", format_time(n).c_str());
	return EXIT_SUCCESS;
}
#endif
